from aa_stripe.models import StripeToken
from aa_stripe.serializers import StripeTokenSerializer
from rest_framework.generics import CreateAPIView
from rest_framework.permissions import IsAuthenticated


class TokensAPI(CreateAPIView):
    queryset = StripeToken.objects.all()
    serializer_class = StripeTokenSerializer
    permission_classes = (IsAuthenticated,)



# -*- coding: utf-8 -*-
from aa_stripe.models import StripeCharge, StripeToken
from django.contrib import admin


class StripeTokenAdmin(admin.ModelAdmin):
    readonly_fields = ("created", "updated", "user", "stripe_js_response", "customer_id")
    list_display = ("id", "user", "created", "is_active")

    def has_delete_permission(self, request, obj=None):
        return False


class StripeChargeAdmin(admin.ModelAdmin):
    readonly_fields = ("created", "updated", "user", "token", "amount", "is_charged", "stripe_charge_id",
                       "description", "comment")
    list_display = ("id", "user", "token", "created", "updated", "is_charged", "amount")

    def has_delete_permission(self, request, obj=None):
        return False


admin.site.register(StripeToken, StripeTokenAdmin)
admin.site.register(StripeCharge, StripeChargeAdmin)



# -*- coding: utf-8 -*-
import stripe
from aa_stripe.models import StripeToken
from django.conf import settings
from rest_framework.exceptions import ValidationError
from rest_framework.serializers import ModelSerializer


class StripeTokenSerializer(ModelSerializer):
    def create(self, validated_data):
        if validated_data.get("stripe_js_response"):
            # Create a Customer
            try:
                user = self.context['request'].user
                stripe.api_key = settings.STRIPE_API_KEY
                stripe_js_response = validated_data.pop("stripe_js_response")
                customer = stripe.Customer.create(
                    source=stripe_js_response["id"],
                    description="{user.first_name} {user.last_name} id: {user.id}".format(user=user)
                )
                instance = StripeToken.objects.create(
                    user=user, customer_id=customer["id"], stripe_js_response=stripe_js_response)
            except stripe.StripeError as e:
                raise ValidationError({"stripe_js_response": e.message})

        return instance

    class Meta:
        model = StripeToken
        fields = ["stripe_js_response"]



# -*- coding: utf-8 -*-
import stripe
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from jsonfield import JSONField


class StripeToken(models.Model):
    """Actually it is Customer. TODO: rename"""

    created = models.DateField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='stripe_tokens')
    stripe_js_response = JSONField()
    customer_id = models.CharField(max_length=255)
    is_active = models.BooleanField(default=True)

    class Meta:
        ordering = ["id"]


class StripeCharge(models.Model):
    created = models.DateField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='stripe_charges')
    token = models.ForeignKey(StripeToken, on_delete=models.SET_NULL, null=True)
    amount = models.IntegerField(null=True, help_text=_("in cents"))
    is_charged = models.BooleanField(default=False)
    stripe_charge_id = models.CharField(max_length=255, blank=True)
    description = models.CharField(max_length=255, help_text=_("Description sent to Stripe"))
    comment = models.CharField(max_length=255, help_text=_("Comment for internal information"))

    def charge(self):
        stripe.api_key = settings.STRIPE_API_KEY
        if not self.is_charged and self.token.is_active:
            try:
                stripe_charge = stripe.Charge.create(
                    amount=self.amount,
                    currency="usd",
                    customer=self.token.customer_id,
                    description=self.description
                )
            except stripe.error.StripeError:
                self.is_charged = False
                self.save()
                raise

            self.stripe_charge_id = stripe_charge["id"]
            self.is_charged = True
            self.save()
            return stripe_charge



from aa_stripe.models import StripeToken


def get_latest_active_token_for_user(user):
    """Returns last active stripe token for user"""
    token = StripeToken.objects.filter(user_id=user.id, is_active=True).last()
    return token



# -*- coding: utf-8 -*-
from aa_stripe.api import TokensAPI
from django.conf.urls import url
from rest_framework.routers import DefaultRouter

urlpatterns = [
    url(r"^aa-stripe/tokens$", TokensAPI.as_view(), name="stripe-tokens"),
]

router = DefaultRouter()
urlpatterns += router.urls
